@@ -101,6 +101,9 @@ struct _GtkHTMLClass {
 					   gfloat position);
 	void     (* cursor_move)          (GtkHTML *html, GtkDirectionType dir_type, GtkHTMLCursorSkipType skip);
 	gboolean (* command)              (GtkHTML *html, GtkHTMLCommandType com_type);
+	void (* cursor_changed)       (GtkHTML *html);
+	void (* object_inserted)       (GtkHTML *html, int pos, int len);
+	void (* object_deleted)        (GtkHTML *html, int pos, int len);
 
 	/* properties */
 	GtkHTMLClassProperties *properties;
@@ -130,7 +133,7 @@ struct _GtkHTMLEditorAPI
 
 /* Creation.  */
 GtkType                    gtk_html_get_type                      (void);
-void                       gtk_html_construct                     (GtkWidget                 *html);
+void                       gtk_html_construct                     (GtkHTML                   *html);
 GtkWidget                 *gtk_html_new                           (void);
 void                       gtk_html_set_editor_api                (GtkHTML                   *html,
 								   GtkHTMLEditorAPI          *api,
@@ -140,6 +143,7 @@ void                       gtk_html_set_editor_api                (GtkHTML
 gint                       gtk_html_set_iframe_parent             (GtkHTML                   *html,
 								   GtkWidget                 *parent,
 								   HTMLObject                *frame);
+GtkHTML                   *gtk_html_get_top_html                  (GtkHTML                   *html);
 
 /* Debugging.  */
 void                       gtk_html_enable_debug                  (GtkHTML                   *html,
@@ -270,6 +274,9 @@ void                       gtk_html_append_html                   (GtkHTML
 /* misc utils */
 gpointer                   gtk_html_get_object_by_id              (GtkHTML                   *html,
 								   const gchar               *id);
+const char *               gtk_html_get_object_id_at              (GtkHTML                   *html,
+								   int                        x,
+								   int                        y);
 gboolean                   gtk_html_command                       (GtkHTML                   *html,
 								   const gchar               *command_name);
 gboolean                   gtk_html_edit_make_cursor_visible      (GtkHTML                   *html);
@@ -319,6 +326,7 @@ gboolean                   gtk_html_save                          (GtkHTML
 								   gpointer                   data);
 GtkHTMLStream             *gtk_html_begin_content                 (GtkHTML                   *html,
 								   gchar                     *content_type);
+void                       gtk_html_drag_dest_set                 (GtkHTML                   *html);
 #endif
 
 #endif /* _GTKHTML_H_ */
