@@ -301,9 +301,14 @@ copy (HTMLObject *self,
 	HTML_CLUEV (dest)->border_color = HTML_CLUEV (self)->border_color;
 	if (HTML_CLUEV (dest)->border_color)
 		html_color_ref (HTML_CLUEV (dest)->border_color);
+	HTML_CLUEV (dest)->background_color = HTML_CLUEV (self)->background_color;
+	if (HTML_CLUEV (dest)->background_color)
+		html_color_ref (HTML_CLUEV (dest)->background_color);
 
 	HTML_CLUEV (dest)->align_left_list = NULL;
 	HTML_CLUEV (dest)->align_right_list = NULL;
+
+	HTML_CLUEV (dest)->dir = HTML_CLUEV (self)->dir;
 }
 
 static gboolean
@@ -426,9 +431,12 @@ draw (HTMLObject *o,
 		if (cluev->border_color) {
 			html_painter_alloc_color (p, &cluev->border_color->color);
 			color = &cluev->border_color->color;
-		} else
-			color = &html_colorset_get_color_allocated (GTK_HTML (p->widget)->engine->settings->color_set,
+		} else {
+			HTMLEngine *e = html_object_engine (o, GTK_HTML (p->widget)->engine);
+			color = &html_colorset_get_color_allocated (e->settings->color_set,
 								    p, HTMLTextColor)->color;
+		}
+
 		html_painter_draw_border (p, color,
 					  tx, ty, 
 					  o->width,
