@@ -28,6 +28,7 @@
 #include <string.h>
 
 #include "config.h"
+#include "htmlcluev.h"
 #include "htmlcolor.h"
 #include "htmlengine.h"
 #include "htmlimage.h"
@@ -195,11 +196,11 @@ html_engine_save_output_string (HTMLEngineSaveState *state,
 }
 
 gboolean
-html_engine_save_output_buffer (HTMLEngineSaveState *state, const gchar *buffer, int len)
+html_engine_save_output_buffer (HTMLEngineSaveState *state, const gchar *buffer, int bytes)
 {
-	if (len == -1)
-		len = strlen (buffer);
-	return state->receiver (state->engine, buffer, len, state->user_data);
+	if (bytes == -1)
+		bytes = strlen (buffer);
+	return state->receiver (state->engine, buffer, bytes, state->user_data);
 }
 
 
@@ -221,7 +222,8 @@ get_body (HTMLEngine *e)
 	gchar *bg;
 	gchar *bg_image;
 	gchar *link;
-	gchar *margins, *lm, *rm, *tm, *bm;
+	char *dir = NULL;
+	gchar *lm, *rm, *tm, *bm;
 	gchar *url = NULL;
 
 	cset = e->settings->color_set;
@@ -233,6 +235,12 @@ get_body (HTMLEngine *e)
 						     (e->widget, ((HTMLImagePointer *) e->bgPixmapPtr)->url))
 		: g_strdup ("");
 	g_free (url);
+	if (HTML_CLUEV (e->clue)->dir != HTML_DIRECTION_DERIVED) {
+		if (HTML_CLUEV (e->clue)->dir == HTML_DIRECTION_RTL)
+			dir = " DIR=RTL";
+		else
+			dir = " DIR=LTR";
+	}
 
 	lm = e->leftBorder != LEFT_BORDER ? g_strdup_printf (" LEFTMARGIN=\"%d\"", e->leftBorder) : g_strdup ("");
 	rm = e->rightBorder != RIGHT_BORDER ? g_strdup_printf (" RIGHTMARGIN=\"%d\"", e->rightBorder) : g_strdup ("");
@@ -394,7 +402,7 @@ html_engine_save_buffer_free (HTMLEngineSaveState *state)
 	string = (GString *)state->user_data;
 
 	g_string_free (string, TRUE);
-	
+
 	g_free (state);
 }
 
@@ -410,7 +418,7 @@ html_engine_save_buffer_peek_text (HTMLEngineSaveState *state)
 }
 
 int
-html_engine_save_buffer_peek_text_len (HTMLEngineSaveState *state)
+html_engine_save_buffer_peek_text_bytes (HTMLEngineSaveState *state)
 {
 	GString *string;
 	
