@@ -26,6 +26,7 @@
 #include <string.h>
 #include <sys/types.h>
 #include <regex.h>
+#include <math.h>
 
 #include <pango/pango.h>
 
@@ -1186,7 +1187,7 @@ prepare_attrs (HTMLText *text, HTMLPainter *painter)
 		flow = HTML_CLUEFLOW (HTML_OBJECT (text)->parent);
 		
 	if (painter->widget && GTK_IS_HTML (painter->widget))
-		e = GTK_HTML (painter->widget)->engine;
+		e = html_object_engine (HTML_OBJECT (text), GTK_HTML (painter->widget)->engine);
 
 	if (flow && e)
 		html_text_add_cite_color (attrs, text, flow, e);
@@ -1196,20 +1197,27 @@ prepare_attrs (HTMLText *text, HTMLPainter *painter)
 		attr->start_index = 0;
 		attr->end_index = text->text_bytes;
 		pango_attr_list_insert (attrs, attr);
-		if (painter->font_manager.fix_size != painter->font_manager.var_size) {
-			attr = pango_attr_size_new (painter->font_manager.fix_size);
+		if (painter->font_manager.fix_size != painter->font_manager.var_size || fabs (painter->font_manager.magnification - 1.0) > 0.001) {
+			attr = pango_attr_size_new (painter->font_manager.fix_size*painter->font_manager.magnification);
+			attr->start_index = 0;
+			attr->end_index = text->text_bytes;
+			pango_attr_list_insert (attrs, attr);
+		}
+	} else {
+		if (fabs (painter->font_manager.magnification - 1.0) > 0.001) {
+			attr = pango_attr_size_new (painter->font_manager.var_size*painter->font_manager.magnification);
 			attr->start_index = 0;
 			attr->end_index = text->text_bytes;
 			pango_attr_list_insert (attrs, attr);
 		}
-	} else
 		pango_attr_list_splice (attrs, text->attr_list, 0, 0);
+	}
 
 	if (text->extra_attr_list)
 		pango_attr_list_splice (attrs, text->extra_attr_list, 0, 0);
 	if (!HTML_IS_PLAIN_PAINTER (painter)) {
 		if (flow && e)
-			html_text_change_attrs (attrs, html_clueflow_get_default_font_style (flow), GTK_HTML (painter->widget)->engine, 0, text->text_bytes, TRUE);
+			html_text_change_attrs (attrs, html_clueflow_get_default_font_style (flow), e, 0, text->text_bytes, TRUE);
 	}
 
 	if (text->links && e) {
